@@ -2,6 +2,6 @@
 ## PLot of the sequences frequency
 ## ================================
 
-seqfplot <- function(seqdata, group = NULL, main = NULL, ...) {
+seqfplot <- function(seqdata, group = NULL, main = "auto", ...) {
 	seqplot(seqdata, group=group, type="f", main=main, ...)
 }
