@@ -2,17 +2,17 @@
 ## Sequences frequency table
 ## =========================
 
-seqstatf <- function(seqdata, weighted=TRUE) {
+seqstatf <- function(seqdata, weighted=TRUE, with.missing=FALSE) {
 
-	if (!inherits(seqdata,"stslist"))
+	if (!is.stslist(seqdata))
 		stop("data is not a sequence object, use seqdef function to create one")
 
-	istatd <- suppressMessages(seqistatd(seqdata))
+	istatd <- suppressMessages(seqistatd(seqdata, with.missing=with.missing))
 
 	## Weights
 	weights <- attr(seqdata, "weights")
 
-	if (!weighted || is.null(weights)) 
+	if (!weighted || is.null(weights))
 		weights <- rep(1, nrow(seqdata))
 
 	istatd <- istatd*weights
@@ -26,5 +26,3 @@ seqstatf <- function(seqdata, weighted=TRUE) {
 	return(res)
 
 }
-
-
